
//Weather Prognosticator
//Version 0.1 (Alpha)
//CC-BY-NC-SA 2016, M. Eric Carr
//eric(at)paleotechnologist.net

//Queries the weatherunderground.com API for hourly temperature data,
//and displays this as colors on a RGB LED strip.

//Based on web client code by Afonso C. Turcato
//(RGB LED strip driver code from Radio Shack website)

//Note: Code is running as-is on an Arduino Uno r3, but seems unstable.
//It is probably about at the memory limits (and/or maybe CPU limit)
//of the Uno. It's probably better suited to a Pi, honestly, but
//the Uno gets the job done for now.

//TODO:
// Query code seems to work most of the time
// Cut the serial output to a minimum or option it out

void(* resetFunc) (void) = 0;//declare reset function at address 0
void reset_strip();
void send_strip(uint32_t data);
void send_1M_pattern(const unsigned long data[][10], int pattern_no, int frame_rate);
void updateFlash();

const int MAXTEMP = 30;
const int MIDTEMP = 15; //Not sure how well having this off-center would work
const int MINTEMP = 0;
const int TEMPRANGE = 30;
const int HALFRANGE = 15;

const unsigned long int QUERYINTERVAL = 300000; //Time between queries in ms.
                                 //Should be at least 172800 to stay under
                                 //the 500 query/day minimum. Probably more.
                                 //Five minutes is 300000, which is a good minimum.

unsigned long colorMap(int temperature);
unsigned long colorMapGBR(int temperature);
unsigned long colorMapNewRGB(int temperature);
unsigned long colorMapNewGBR(int temperature);
int temperature[36]; //

unsigned long lightArray[10]; //Holds the colors for quick lookup

int *tempPointer = &temperature[0];  //Hopefully functions can use this to access it

#include "JsonStreamingParser.h"
#include "JsonListener.h"
#include "ExampleParser.h"

//RGB LED stuff
#include <avr/pgmspace.h>
#define DATA_1 (PORTC |=  0X01)    // DATA 1    // for UNO
#define DATA_0 (PORTC &=  0XFE)    // DATA 0    // for UNO
#define STRIP_PINOUT (DDRC=0xFF)    // for UNO


unsigned char startCapture;

unsigned long int lastRead;

JsonStreamingParser parser;
ExampleListener listener;

/*
 Web Client to consume Weather Underground web service
 
 This sketch connects to a website (http://api.wunderground.com)
 using an Arduino Ethernet shield and get data from site.
 
 Circuit:
 * Arduino MEGA 2560 R3 Board
 * Ethernet shield attached to pins 10, 11, 12, 13
 
 created 07 Jan 2014
 by Afonso C. Turcato


 2016-07-30
 ----------
 Modified to use a streaming JSON parser to handle 36-hour forecasts, to extract
 hourly temperature data for display on a RGB LED strip.
 M. Eric Carr / eric@paleotechnologist.net
 

 
 */

#include <SPI.h>
#include <Ethernet.h>

// Enter a MAC address for your controller below.
// Newer Ethernet shields have a MAC address printed on a sticker on the shield
byte mac[] = { 0xFE, 0xED, 0xDE, 0xAD, 0xBE, 0xEF };

// if you don't want to use DNS (and reduce your sketch size)
// use the numeric IP instead of the name for the server:
IPAddress server(23,216,168,120);  // numeric IP for Weather Underground  (no DNS)
//const char server[] = "api.wunderground.com";    // name address for Weather Underground (using DNS)

const String myKey PROGMEM = "xxxxxxxxxxxxxxxx";  //See: http://www.wunderground.com/weather/api/d/docs (change here with your KEY)
const String myFeatures PROGMEM = "hourly";   //See: http://www.wunderground.com/weather/api/d/docs?d=data/index&MR=1
const String myCountry PROGMEM = "PA";        //See: http://www.wunderground.com/weather/api/d/docs?d=resources/country-to-iso-matching
const String myCity PROGMEM = "Philadelphia";      //See: http://www.wunderground.com/weather/api/d/docs?d=data/index&MR=1

const String html_cmd1 PROGMEM = "GET /api/" + myKey + "/" + myFeatures + "/q/" + myCountry + "/" + myCity + ".json HTTP/1.1";
const String html_cmd2 PROGMEM = "Host: api.wunderground.com";
const String html_cmd3 PROGMEM = "Connection: close";


unsigned long byteCount;
unsigned char connState;

unsigned char currentTemp;

// Set the static IP address to use if the DHCP fails to assign
IPAddress ip(192,168,0,177);

// Initialize the Ethernet client library
// with the IP address and port of the server 
// that you want to connect to (port 80 is default for HTTP):
EthernetClient client;

//Response from Server
//String responseString;

//unsigned char captureState;

void setup() {

  pinMode(A5, INPUT);
  digitalWrite(A5, HIGH);

  //RGB LED stuff
  STRIP_PINOUT;        // set output pin - DEBUG: should auto detect which mother board for use
  reset_strip();

  
  // Open serial communications and wait for port to open:
  Serial.begin(57600);

  //Serial.print("Starting.");

  //Set up the JSON parser...?
  parser.setListener(&listener);
  byteCount = 0;
  connState = 0;
  startCapture = 0;
  
  for(currentTemp=0;currentTemp<36;currentTemp++){
    temperature[currentTemp]=0;
    }
  currentTemp = 0;

  lastRead = 0; //Time in ms of last successful read

  // start the Ethernet connection:
  Ethernet.begin(mac, ip);

Serial.println("Ethernet done.");
  
  // give the Ethernet shield a half-second to initialize:
  delay(500);
  Serial.print("IP: ");
  Serial.println(Ethernet.localIP());

  Serial.println("Done setup.");
}

void loop(){
    char c;

  //Moved all the http connect stuff here so it can loop **EC**
  
  Serial.println("Connecting...");
  
  // if you get a connection, report back via serial:
  if (client.connect(server, 80)){
    connState = 1;
    Serial.println("Connected!");
    
    // Make a HTTP request:
    client.println(html_cmd1);
    client.println(html_cmd2);
    client.println(html_cmd3);
    client.println();

    startCapture = false;   
  } 
  else
  {
    // if you didn't get a connection to the server:
    Serial.println("Connect fail!");
    delay(500);
    connState = 3; //Failed
    //while(1);  //HALT
  }

    Serial.println("reading.");

    currentTemp = 0;

  while(client.connected() && connState == 1){

  // if there are incoming bytes available 
  // from the server, read them and buffer:
  while (client.available())
  {
    c = client.read();
    //Serial.print(c);
    //Serial.print("*");

    //PARSER GOES HERE
    parser.parse(c);    //That was easy.
    byteCount++;
    
  }//while

  
  // if the server's disconnected, stop the client:
  if (connState == 1 && (!client.connected() || currentTemp > 9)) {    
    connState = 2;  //ONLY DO THIS ONCE.
    //Serial.println("Received " + (String)byteCount + " bytes");
    //Serial.println("Disconnecting.");
    
    Serial.println();

    for(c=0;c<10;c++){
      Serial.print((String)(int)c);
      Serial.print(" ");
      Serial.print(temperature[c]);
      //Serial.print(c*4);
      Serial.print(" ");
      Serial.println(colorMapNewRGB(temperature[c]), HEX);
      //Serial.println(colorMapNewRGB(c*4), HEX);
      }

    //Compute the colors for quick sending
    for(c=0;c<10;c++){
      lightArray[c] = colorMapNewGBR(temperature[c]);
      //lightArray[c] = colorMapNewGBR(c*4);
      }

    //Flash and then clear the display to signify new data loaded
    //updateFlash();

    noInterrupts();
    reset_strip();
    for(c=0;c<10;c++){
      send_strip(lightArray[9-c]);  //send LED color in reverse order
      }//for
    interrupts();
    connState = 2;  //Clean read.
    lastRead = millis();
    
    }//if

}//while connected

  //If disconnected, wait and go again.
  
  if (connState != 3){
    Serial.print ("Waiting ");
    Serial.print (QUERYINTERVAL/1000);
    Serial.println(" s");
    while(lastRead + QUERYINTERVAL > millis()){
      delay(100); //No need to be super aggressive here.
                  //Data is 1hr apart, so reads every 10m are overkill
                  //and exceeding that by 100ms is nothing.
      }//while 
  }//if
  connState = 0;
  client.stop();
  client.flush();

  
  // **HACK HACK UGLY HACK**
  // (...but it's a *working* hack, so don't break it.)

  resetFunc(); //call reset by executing address zero.
  
  delay(5000);  //Not that you're still executing this code, but...

  Serial.println("Reset didn't work!"); //Haven't seen this yet
    
  }//loop()


unsigned char coerce(int num){
  if (num<0){return(0);}
  if (num>255){return(255);}
  return(num);
}

unsigned long colorMapNewGBR(int temperature){
 
  int red, green, blue;
  unsigned char RR, GG, BB;
  //unsigned long temp;

/*

  //Pass the buck to avoid writing the same routine twice.
  //We'll reshuffle the bits here.
  temp = colorMapNewRGB(temperature);
  
  RR = (temp & 0xFF0000) >> 16;
  GG = (temp & 0x00FF00) >> 8;
  BB = (temp & 0x0000FF);

  temp = (GG << 16) | (BB << 8) | (RR);

  return(temp);
  
  }

*/
  
  unsigned long result;
  
  red = 255 * (temperature-MIDTEMP)/HALFRANGE;
  RR = coerce(red);
  green = 255-255*abs(temperature-MIDTEMP)/HALFRANGE;
  GG = coerce(green);
  blue = 255 * (MIDTEMP-temperature)/HALFRANGE;
  BB = coerce(blue);
  
  result = 0;
  result |= (GG & 0xFF);
  result = result << 8;
  result |= (BB & 0xFF);
  result = result << 8;
  result |= (RR & 0xFF);

  return(result);
  
  }


unsigned long colorMapNewRGB(int temperature){
  
  int red, green, blue;
  unsigned char RR, GG, BB;

  unsigned long result;

  
  red = 255 * (temperature-MIDTEMP)/HALFRANGE;
  RR = coerce(red);
  green = 255-255*abs(temperature-MIDTEMP)/HALFRANGE;
  GG = coerce(green);
  blue = 255 * (MIDTEMP-temperature)/HALFRANGE;
  BB = coerce(blue);
  
  result = 0;
  result |= (RR & 0xFF);
  result = result << 8;
  result |= (GG & 0xFF);
  result = result << 8;
  result |= (BB & 0xFF);
  
  return(result);
  
  }


/*******************************************************************************

* Function Name  : send_strip

* Description    : Transmit 24 pulse to LED strip

*                  

 * Input          : 24-bit data for the strip

*                  

 * Output         : None

* Return         : None

*******************************************************************************/

void send_strip(uint32_t data)

{

  int i;

  unsigned long j=0x800000;

  for (i=0;i<24;i++){
    if (data & j) {

      DATA_1;
      __asm__("nop\n\t");
      __asm__("nop\n\t");
      __asm__("nop\n\t");
      __asm__("nop\n\t");
      __asm__("nop\n\t");
      __asm__("nop\n\t");
      __asm__("nop\n\t");
      __asm__("nop\n\t");
      __asm__("nop\n\t");    
      __asm__("nop\n\t");
      __asm__("nop\n\t");
      __asm__("nop\n\t");
      __asm__("nop\n\t");
      __asm__("nop\n\t");
      __asm__("nop\n\t");
      __asm__("nop\n\t");
      __asm__("nop\n\t");
      __asm__("nop\n\t");

/*----------------------------*/
      __asm__("nop\n\t");
      __asm__("nop\n\t");
      __asm__("nop\n\t");
      __asm__("nop\n\t");
      __asm__("nop\n\t");  
      __asm__("nop\n\t");
      __asm__("nop\n\t");
      __asm__("nop\n\t");  
      __asm__("nop\n\t");  
      __asm__("nop\n\t");        

/*----------------------------*/      

      DATA_0;

    }

    else

    {

      DATA_1;

      __asm__("nop\n\t");
      __asm__("nop\n\t");
      __asm__("nop\n\t");
      __asm__("nop\n\t");
      __asm__("nop\n\t");
      __asm__("nop\n\t");
      __asm__("nop\n\t");
      __asm__("nop\n\t");
      __asm__("nop\n\t");    

      DATA_0;

/*----------------------------*/      

      __asm__("nop\n\t");
      __asm__("nop\n\t");
      __asm__("nop\n\t");      

/*----------------------------*/         

    }

 

    j>>=1;

  }

 

 

 

}

 

/*******************************************************************************

* Function Name  : reset_strip

* Description    : Send reset pulse to reset all color of the strip

*                  

 * Input          : None

*                  

 * Output         : None

* Return         : None

*******************************************************************************/

void    reset_strip()

{

  DATA_0;

  delayMicroseconds(20);

}



void updateFlash(){
  int a,b;
  for(a=9;a>-2;a--){
    noInterrupts();
    for(b=0;b<10;b++){
      if(a==b){
        send_strip(0xFFFFFF);
        }
      else{
        send_strip(lightArray[9-b]);
        }
      }//for b     
    interrupts(); 
    delay(50);
    }//for a
  }


